#!/usr/bin/perl -w
use strict;
use Socket;
use POE qw(Wheel::SocketFactory Wheel::ReadWrite);

for my $port (@ARGV) {
  POE::Session->create
      (args => $port,
       inline_states => {
           _start	=> \&start,
           _default	=> \&default,

           connected	=> \&connected,
           connect_failed => \&connect_failed,

           got_line	    => \&got_line,
           connection_error => \&connection_error,
       });
}
$poe_kernel->run();

sub default {
    print STDERR "calling non existant event $_[ARG0]\n" unless
        substr($_[ARG0], 0, 1) eq "_";
    return;
}

sub start {
    my ($heap, $port) = @_[HEAP, ARG0];
    $heap->{wheel} = POE::Wheel::SocketFactory->new
        (RemoteAddress => "127.0.0.1",
         RemotePort    => $port,
         SuccessEvent  => "connected",
         FailureEvent  => "connect_failed");
}

sub connect_failed {
    my ($heap, $operator, $errstr) = @_[HEAP, ARG0, ARG2];
    print STDERR "$operator: $errstr\n";
    delete $heap->{wheel};
}

sub connected {
    my ($heap, $socket) = @_[HEAP, ARG0];
    $heap->{wheel} = POE::Wheel::ReadWrite->new
        (Handle     => $socket,
         InputEvent => "got_line",
         ErrorEvent => "connection_error");
}

sub connection_error {
    my ($heap, $operator, $errno, $errstr, $wheel_id) = @_[HEAP, ARG0..ARG3];
    delete $heap->{wheel};
    if ($errno) {
        print STDERR "$operator: $errstr\n";
    } else {
        # EOF
        print STDERR "Connection closed\n";
    }
}

sub got_line {
    my ($heap, $line) = @_[HEAP, ARG0];
    if ($line =~ /Number/) {
        $heap->{wheel}->put(int rand 10);
    } elsif (my ($result) = $line =~ /=(\d+)/) {
        printf STDERR "wheel %d: $line\n", $heap->{wheel}->ID;
        if ($result == 42) {
            print STDERR "Finally!\n";
            delete $heap->{wheel};
        }
    } else {
        delete $heap->{wheel};
        print STDERR "Unexpected line '$line'\n";
    }
}
