#!/usr/bin/perl -w
use strict;
use POSIX qw(EINTR _exit);
use IO::Socket::INET;

$SIG{CHLD} = "IGNORE";
$SIG{PIPE} = "IGNORE";

defined(my $file = shift) || die "Usage: $0 file\n";

my $socket = IO::Socket::INET->new(LocalAddr => "127.0.0.1",
                                   LocalPort => 1234,
                                   Listen    => 5) ||
    die "Could not listen on 127.0.0.1:1234: $!\n";

while (1) {
    my $accepted = $socket->accept;
    # (my ($accepted, $peer_packed) = $socket->accept;
    if (!$accepted) {
        print STDERR "Error accepting connection: $!\n" unless 
            $! == EINTR;
        next;
    }
    defined(my $pid = fork) ||
        die "Could not fork: $!";

    if ($pid) {
        # Parent
        # Schrijf expliciet voor duidelijkheid
        close $accepted;
        next;
    }
    # Child
    close $socket;
    my $peer_port = $accepted->peerport;
    my $peer_ip   = $accepted->peerhost;
    print STDERR "Pid $$, received a connection from $peer_ip:$peer_port\n";

    local $_;
    defined($pid = fork) ||
        die "Could not fork: $!";
    if ($pid) {
        # parent
        shutdown($accepted, 1);
        1 while <$accepted>;
    } else {
        # child
        shutdown($accepted, 0);
        open(my $fh, "<", $file) || die "Could not open $file: $!";
        print $accepted $_ while <$fh>;
    }
    close $accepted;
    print STDERR "Next!\n";
    _exit(0);
}
