#!/usr/bin/perl -w
use strict;
use POSIX ":sys_wait_h";
use IO::Socket::INET;

sub reaper {
    my $child;
    # If a second child dies while in the signal handler caused by the
    # first death, we won't get another signal. So must loop here else
    # we will leave the unreaped child as a zombie. And the next time
    # two children die we get another zombie. And so on.
    while ((my $child = waitpid(-1,WNOHANG)) > 0) {
        print STDERR "Child $child exits with status $?\n";
    }
    $SIG{CHLD} = \&reaper;  # loathe sysV
}
$SIG{CHLD} = \&reaper;

# Modern UNIX:
# $SIG{CHLD} = "IGNORE";

my $socket = IO::Socket::INET->new(LocalAddr => "127.0.0.1",
                                   LocalPort => 1234,
                                   Listen    => 5) ||
    die "Could not listen on 127.0.0.1:1234: $!\n";

while (1) {
    my $accepted = $socket->accept;
    # (my ($accepted, $peer_packed) = $socket->accept;
    if (!$accepted) {
        print STDERR "Error accepting connection: $!\n";
        next;
    }
    defined(my $pid = fork) ||
        die "Could not fork: $!";

    if ($pid) {
        # Parent
        # Schrijf expliciet voor duidelijkheid
        close $accepted;
        next;
    }
    # Child
    close $socket;
    my $peer_port = $accepted->peerport;
    my $peer_ip   = $accepted->peerhost;
    print STDERR "Pid $$, received a connection from $peer_ip:$peer_port\n";
    print $accepted "Hello $peer_ip:$peer_port\n";
    sleep 5;
    close $accepted;
    exit;
}
